#include <thread>

#include "hpp/dds/dds.hpp"
#include "IDL/IDLS.hpp"

int main(int argc, char *argv[])
{

    // creating Domain Participant
    dds::domain::qos::DomainParticipantQos dpQos = dds::core::QosProvider::Default()
            .participant_qos(rti::core::builtin_profiles::qos_lib::baseline());
    rti::core::policy::DomainParticipantResourceLimits resource_limits_qos;
    resource_limits_qos.type_code_max_serialized_length(0);
    resource_limits_qos.reader_user_data_max_length(65536);
    resource_limits_qos.writer_user_data_max_length(65536);
    resource_limits_qos.type_object_max_serialized_length(20000);
    dpQos << resource_limits_qos;
    std::map<std::string,std::string> dpPropertyMap = {{"dds.transport.UDPv4.builtin.recv_socket_buffer_size", "16777216"},
                                                       {"dds.transport.UDPv4.builtin.send_socket_buffer_size", "16777216"},
//                                                       {"dds.transport.UDPv4.builtin.parent.message_size_max", "100000"},
                                                       {"dds.transport.shmem.builtin.receive_buffer_size", "2048576"}};

    dpQos << rti::core::policy::Property(dpPropertyMap.begin(),dpPropertyMap.end());

    dds::domain::DomainParticipant domainParticipant(0,dpQos);

    // choose a topicName
    std::string topicName = "Test_Recording_Service_Message";

    // create topic with the domainParticipant, the topicName, the DataType
    dds::topic::Topic<Structs::SendMessage> topic(domainParticipant,topicName);;

    // create subscriber with created domainParticipant
    dds::sub::Subscriber subscriber(domainParticipant);

    // create DataReaderQos object and set Reliability Qos and History Qos
    dds::sub::qos::DataReaderQos dataReaderQos = domainParticipant->default_datareader_qos();
    dataReaderQos << dds::core::policy::Reliability::Reliable()
                  << dds::core::policy::History::KeepAll();

    std::string userDataString = "this is my user data";
    std::vector<uint8_t> userDataVec(userDataString.begin(), userDataString.end());
    dds::core::policy::UserData userData(userDataVec);
    dataReaderQos->user_data = userData;

    // create DataReader opject with the subscriber, the topic, and the dataReaderQos
    dds::sub::DataReader<Structs::SendMessage> dataReader(subscriber, topic, dataReaderQos);

    // take any received data for 2 minutes
    if (dataReader.is_nil() == false)
    {
        auto start = std::chrono::high_resolution_clock::now();
        long difference = 0;

        while(difference < 120000)
        {
            // Taking data from data reader
            auto lnSamples = dataReader->take();
            if (lnSamples.length() == 0)
            {
                std::this_thread::sleep_for(std::chrono::milliseconds(100));
            }

            for (const auto& index : lnSamples)
            {
                if (index.info().valid())
                {
                    if (index.data().textType() == Enums::TextType::Hint)
                    {
                            std::cerr << "Hint: ";
                    }
                    else if (index.data().textType() == Enums::TextType::Error)
                    {
                            std::cerr << "Error: ";
                    }
                    else if (index.data().textType() == Enums::TextType::Warning)
                    {
                            std::cerr << "warning: ";
                    }

                    std::cerr << index.data().textString() << std::endl;
                }
            }

            auto now = std::chrono::high_resolution_clock::now();
            difference = std::chrono::duration_cast<std::chrono::milliseconds>(now - start).count();
        }
    }
}
