#include <thread>

#include "hpp/dds/dds.hpp"
#include "IDL/IDLS.hpp"

int main(int argc, char *argv[])
{

    // creating Domain Participant
    dds::domain::qos::DomainParticipantQos dpQos = dds::core::QosProvider::Default()
            .participant_qos(rti::core::builtin_profiles::qos_lib::baseline());
    rti::core::policy::DomainParticipantResourceLimits resource_limits_qos;
    resource_limits_qos.type_code_max_serialized_length(0);
    resource_limits_qos.reader_user_data_max_length(65536);
    resource_limits_qos.writer_user_data_max_length(65536);
    resource_limits_qos.type_object_max_serialized_length(20000);
    dpQos << resource_limits_qos;
    std::map<std::string,std::string> dpPropertyMap = {{"dds.transport.UDPv4.builtin.recv_socket_buffer_size", "16777216"},
                                                       {"dds.transport.UDPv4.builtin.send_socket_buffer_size", "16777216"},
//                                                       {"dds.transport.UDPv4.builtin.parent.message_size_max", "100000"},
                                                       {"dds.transport.shmem.builtin.receive_buffer_size", "2048576"}};

    dpQos << rti::core::policy::Property(dpPropertyMap.begin(),dpPropertyMap.end());

    dds::domain::DomainParticipant domainParticipant(0,dpQos);

    // choose a topicName
    std::string topicName = "Test_Recording_Service_Message";

    // create topic with the domainParticipant, the topicName, the DataType
    dds::topic::Topic<Structs::SendMessage> topic(domainParticipant,topicName);

    // create publisher with created domainParticipant
    dds::pub::Publisher publisher(domainParticipant);

    // create DataWriterQos object and set Reliability Qos, History Qos, and Lifespan Qos
    dds::pub::qos::DataWriterQos dataWriterQos = domainParticipant->default_datawriter_qos();
    dataWriterQos << dds::core::policy::Reliability::Reliable()
                  << dds::core::policy::History::KeepAll()
                  << dds::core::policy::Lifespan(dds::core::Duration(10,0));

    // create DataWriter opject with the publisher, the topic, and the dataWriterQos
    dds::pub::DataWriter<Structs::SendMessage> dataWriter(publisher, topic, dataWriterQos);

    // wait for connecting to TEXT_RECEIVER (if dont, samples will not receive)
    std::this_thread::sleep_for(std::chrono::seconds(20));

    // create an object with the data type for sending sample
    Structs::SendMessage sample;

    // fill the object with correct value
    sample.textType(Enums::TextType::Hint);
    sample.textString(std::string("This is a Hint;"));

    for(int i=0; i < 5; i++)
    {
        // send the object
        dataWriter.write(sample);
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }

    sample.textType(Enums::TextType::Error);
    sample.textString(std::string("This is a Error;"));
    for(int i=0; i < 5; i++)
    {
        dataWriter.write(sample);
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }

    sample.textType(Enums::TextType::Warning);
    sample.textString(std::string("This is a Warning;"));
    for(int i=0; i < 5; i++)
    {
        dataWriter.write(sample);
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }

}
